<!DOCTYPE style-sheet PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN">

;; Toby's Empeg-playlist stylesheet, version 1 - August 2002
;; Copyright 2002 Toby Speight
;; Author: Toby Speight <streapadair@gmx.net>

;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This software is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.


;; User configuration:

;; To override these variables using OpenJade, add "-V '(define
;; %variable value)'" to the command-line (including the single
;; quotes), or for boolean variables, simply "-V %variable".

(define %scale-4up #f)          ; Override one of these for convenient
(define %scale-2up #f)          ; setting of %scale.

(define %scale                  ; Scale font-sizes, indents etc -
  (cond (%scale-4up 2)          ; useful if the output is to be
        (%scale-2up (sqrt 2))   ; reduced later.
        (else 1)))



(define %root-element-id "x100") ; Override to print a different playlist.

(define %three-columns #f)      ; Override one of these for
(define %two-columns #f)        ; convenient setting of %column.

(define %columns
  (cond (%three-columns 3)
        (%two-columns 2)
        (else 1)))              ; Override for multicolumn output.

(define %font-size (* 8pt %scale))
(define %child-indent %font-size)
(define %continuation-indent (/ %font-size 2))


;; The following parameters control which formatting is chosen for
;; playlists with print=normal (the default).  To override these
;; heuristics on a per-playlist basis, set print={none,summary,all}
;; as appropriate.  print=none also applies to tunes and will cause
;; them to completely disappear from the printout (even from summary
;; counts).

; Max no of tunes in a tune-only playlist before summary mode is used.
(define %tune-limit 1)

; Max no of tunes in a mixed playlist before summary mode is used.
(define %mixed-tune-limit %tune-limit)

; In mixed playlists, ignore this proportion of the playlists in addition
; to %mixed-tune-limit.  Playlists with lots of sub-playlists are therefore
; also able to show lots of individual tunes, too.  I like to set this
; somewhere between 0.5 and 1.0.
(define %mixed-tune-ratio 0.0)


;;; No user-servicable parts beyond this line.

(root
    (make simple-page-sequence
      page-width:    208mm
      page-height:   284mm
      page-n-columns: %columns
      left-margin:   0.5cm
      right-margin:  1cm
      top-margin:    1cm
      bottom-margin: 1cm
      font-size:     %font-size
      (process-element-with-id  %root-element-id)))


(element playlist
  (let* ((raw-content (referent (node-list-property 'value (attribute "content" (current-node)))))
         (content (node-list-filter printable? raw-content))
         (playlists (select-elements content '(playlist)))
         (playlist-count (node-list-length playlists))
         (tunes (select-elements content '("tune")))
         (tune-count (node-list-length tunes))
         (title-attrib (select-elements (children (current-node)) '("attribute" ("name" "title"))))
         (title (data title-attrib)))
    (case (print-attrib (current-node))
      ((none no) (empty-sosofo))
      ((summary) (print-summary title-attrib (count-tunes content)))
      ((playlists) (print-playlists title-attrib playlists tune-count))
      ((all) (print-all title-attrib content))
      ((normal)
       (if (node-list-empty? playlists)
           (cond ;; No playlists
            ((node-list-empty? tunes) (empty-sosofo))
            ((<= tune-count %tune-limit)
             (if (equal? 1 tune-count)
                 (print-one-liner title-attrib tunes)
               (print-all title-attrib content)))
            (else (print-summary title-attrib tune-count)))
         (cond ;; some playlists
          ((<= tune-count (+ (* %mixed-tune-ratio playlist-count) %mixed-tune-limit))
           (print-all title-attrib content))
          (else
           (print-playlists title-attrib playlists tune-count))))))))

(define (print-all title-attrib content)
  (sosofo-append
   ;; Heading
   ;;  child playlists and tunes
   (make paragraph
     first-line-start-indent: (- %continuation-indent)
     start-indent: (+ %continuation-indent (inherited-start-indent))
     font-weight: 'bold
     (process-node-list title-attrib))
   (make paragraph
     keep-with-previous?: #t
     start-indent: (+ (inherited-start-indent) %child-indent)
     (process-node-list (node-list-first content)))
   (make paragraph
     start-indent: (+ (inherited-start-indent) %child-indent)
     (process-node-list (node-list-rest content)))))

(define (print-playlists title-attrib playlists tune-count)
   ;; Heading
   ;;  child playlists
   ;;  tune count
  (sosofo-append
   (make paragraph
     first-line-start-indent: (- %continuation-indent)
     start-indent: (+ %continuation-indent (inherited-start-indent))
     font-weight: 'bold
     (process-node-list title-attrib))
   (make paragraph
     keep-with-previous?: #t
     start-indent: (+ (inherited-start-indent) %child-indent)
     (process-node-list (node-list-first playlists)))
   (make paragraph
     start-indent: (+ (inherited-start-indent) %child-indent)
     (process-node-list (node-list-rest playlists))
     (make paragraph
       keep-with-previous?: #t
       (literal (number->string tune-count) " more")))))

(define (print-one-liner title-attrib tune)
  ;; Heading: tune
  (make paragraph
    first-line-start-indent: (- %continuation-indent)
    start-indent: (+ %continuation-indent (inherited-start-indent))
    (make sequence
      font-weight: 'bold
      (process-node-list title-attrib))
    (literal ": " (data (select-elements (children tune) '("attribute" ("name" "title")))))))

(define (print-summary title-attrib subtree-count)
  ;; Heading (count)
  (make paragraph
    first-line-start-indent: (- %continuation-indent)
    start-indent: (+ %continuation-indent (inherited-start-indent))
    (make sequence
      font-weight: 'bold
      (process-node-list title-attrib))
    (literal " (" (number->string subtree-count) ")")))


(define (count-tunes nl)
  (letrec ((count (lambda (n nl)
                    (if (node-list-empty? nl)
                        n
                      (let* ((first (node-list-first nl))
                             (rest (node-list-rest nl))
                             (childrefs (node-list-property 'value (attribute "content" first))))
                        (if (node-list-empty? childrefs)
                            (count (+ 1 n) rest)
                          (count
                           (+ n (count 0 (node-list-filter printable? (referent childrefs))))
                           rest)))))))
    (count 0 nl)))

;; Utilities

(define (printable? nl)
  (let ((tag (print-attrib nl)))
    (case tag
          ((all normal playlists summary) #t)
          ((none no) #f)
          (else (node-list-error "Invalid 'print' property value" nl)))))

(define (print-attrib nl)
  ;; #t if we should print this node
  ;; i.e. its "print" property is true or missing
  (let ((print-attr (select-elements (children nl) '("attribute" ("name" "print")))))
    (if (node-list-empty? print-attr)
        'normal
        (string->symbol (data print-attr)))))

(element tune
  (make paragraph
    (process-node-list (q-element '("attribute" ("name" "title"))))))


(declare-characteristic
 page-n-columns "UNREGISTERED::James Clark//Characteristic::page-n-columns" 1)

;; Override OpenJade's b0rked implementation
(define (attribute name nl)
  (node-list-map (lambda (snl)
		   (named-node name (attributes snl)))
		 nl))


(define debug
  (or (external-procedure "UNREGISTERED::James Clark//Procedure::debug")
      (lambda (x) x)))
